class Database:
    """Constants related to databases
    """
    # Database table names
    skater_stats_table_name = "skater_stats"
    goalie_stats_table_name = "goalie_stats"
    players_table_name = "players"
    games_table_name = "games"
    meta_table_name = "meta"